html, body {
  height: 100%;
  margin: 0;
}
body {
  background-color: #fafafa !important;
  font-family: sans-serif;
  overflow: hidden;
}
a{
-moz-user-select: none; -webkit-user-select: none; -ms-user-select:none; user-select:none;-o-user-select:none;

}

#blocklyArea {
  height: 90%;
}
.navbar {
  background-color: #274264;
}

.welcome_option:hover {
opacity: .4;
}


code {
  font-family: monospace;
  white-space: pre-wrap;
  color: #939393 !important;
}

#q_panel {
  color: #5a5a5a;
  overflow-y:auto;
  height: calc(100% - 80px); 
  padding-bottom: 100px;
}

#title {
  background-color: transparent !important;
  border: 0px;
  outline: none;
  color: white;
  font-size: 22px;
min-width: 80px;
max-width: 30%;
height: 32px;
padding-top: 0px !important;
}

#title:focus {
  text-decoration: underline;
}

.submit-btn {
  border-color: white!important;
  color: white!important;
  border-radius: .2rem!important;
}

.navbar-dark, .navbar-brand {
  font-family: Google sans;
    font-size: 23px;
    color: #fcfcfc;
}

#blockly_panel {
  padding: 0px!important;
}

.ro {
  display: table;
  table-layout: fixed;
  width: 100%;
  height: 100%
}

.left{
  left: -15px;
  display: table-cell;
  vertical-align: top;
  position: relative;
  width: 66.6%;
}

.right {
  overflow: hidden;
  display: table-cell;
  vertical-align: top;
  width:33.3%;
  position: relative;
}

.right_content {
  height: 100vh;
  overflow-y: auto;

}

.submit_bar {
  position: absolute;
  padding: 5px;
  padding-right: 20px;
  bottom:50px;
  width: 100%;
  background-color: #f1f1f1;
  height: 50px;
  right: 0px;
}