html, body, #fullheight {
  position: relative;
  min-height: 100% !important;
  height: 100%;
  font-family: "Open Sans", "Roboto", sans-serif !important;
  background-color: #fafafa !important;

}

body {
  overflow-y: scroll!important;

}

h5 {
  font-weight: 400!important;
}

.navbar.navbar-dark {
  font-size: 17px!important;
}

#main {
padding-top: 55px;
}

#navbar {
  background-color: #1a73e8;
  z-index: 10;
  box-shadow: 0 2px 5px 0 rgba(255, 255, 255, 0.16)!important;
}

#container, #main{
  position: relative;
  width: 98vw;
  top: 0px;
  height: 100%;
  z-index: 0;
}

#sidenav {
  padding: 0px;
  height: 100%;
  background-color:  #FFFFFF!important;
  border-right: 1px solid rgba(0,0,0,.12);
}


.list-group{
  padding-top: 20px;
}

.list-group-item.active {
  background-color:  #E8F0FE!important;
  font-weight: 500;
  color: #1E6AD3!important;
}

.list-group-item {
  font-size: 15px;
  font-family: "Roboto Bold";
  font-weight: normal;
  color: #495057;
  background-color:  transparent!important;
  padding-right: 20px;
  border-top-right-radius: 10rem!important;
  border-bottom-right-radius: 10rem!important;
  border: 0px!important;
  width:15vw!important;
}
.list-group-item:hover {

 background-color: #F5F5F5!important;
}

.sidebar-search {
  padding-top: 10px;
}

.md-form {
  width: 95%
}

input {
  width: 100%!important;
}

.tab {
  padding-top: 10px;
  width: 100%;
}

.list-group-flush .list-group-item {
  width: 100%;
}

#map {
  min-height: 400px;
}

.box {
  box-shadow: 0 1px 1px 0 rgba(60,64,67,.02), 0 1px 3px 1px rgba(60,64,67,.1);
  width: 100%;
  padding:10px;
  margin: 10px;
  min-height: 100px;
  height: 450px;
  background-color: white;
}

.btn {
  -webkit-box-shadow: 0 0px 0px 0 rgba(0,0,0,.16), 0 2px 10px 0 rgba(0,0,0,.0)!important;
  box-shadow: 0 0px 0px 0 rgba(0,0,0,.16), 0 2px 10px 0 rgba(0,0,0,.0)!important;
}

.btn.btn-md {
  font-size: 14px!important;

}

#map {
  height: 100%;
  width: 100%;
}

.frame{
  width: 100%;
  border-radius: 4px;

}
img {
  width: 100%;
  border-radius: 4px;
}

#frame {
height: auto;
}


#sidenav {
position: fixed!important;
left: 0px;
top:55px;
z-index: 5;

}


.c-card {
  background-color: white;
  border: solid 1px #e5e5e5;
  border-radius: .5rem;
  margin: 30px;
  padding: 20px;
  box-shadow: 0px!important;
}

.c-card-file {
  background-color: white;
  padding: 15px;
  margin-top: 10px;
  margin-bottom: 10px;
  border: solid 1px #e5e5e5;
}

.c-card-file:hover {
  border-color: rgb(211, 211, 211);
  background-color: rgb(250, 250, 250);
  cursor: pointer;
}

p {
  font-family: Roboto Bold;
  font-size: 14px;
  color: #505050;

}

.light {
  color: #7e7e7e;
}

.c-card:hover {
  border-color: rgb(211, 211, 211);
  background-color: rgb(250, 250, 250);
  cursor: pointer;
}

.modal-content {
  border-radius: 8px!important;
}

.modal-footer {
  border-top: 0px!important;
}

.modal-header {
  border-bottom: 0px!important;
}

button {
  border-radius: 4px!important;
}

.pwd {
  background:#E8F0FE;
  border: solid 1px #e4e4e4;
  border-radius: 5px;
  height: auto;
  margin-top: 18px;
  padding: 8px;
  display: flex;
  flex-direction: row;
  justify-content: space-between;
}

.pwd-txt {
  font-family: "Open Sans" !important;
  vertical-align: middle;
  margin-top: auto;
  margin-bottom: auto;
  color: #0056b3;
  margin-left: 10px;
}

.new-btn {
  padding: 5px!important;
  padding-left: 10px!important;
  padding-right: 10px!important;
  margin: 0px!important;
  background-color:#E8F0FE!important;
  color: #0056b3!important;
}

.new-btn:hover {
  background-color:rgb(212, 228, 255)!important;
}

.menu {
  width: 120px;
  box-shadow: -1px 2px 15px 5px rgba(0, 0, 0, 0.1),0 0 2px 0px rgba(0, 0, 0, 0.1);
  position: absolute;
  display: none;
  z-index: 10 ;
  background-color: white;
  color: #505050;
  transition: display 2s linear;
}

.menu-options {
    list-style: none;
    padding: 10px 0;
    margin: 0px;
}
.menu-option {
    font-weight: 500;
    font-size: 14px;
    padding: 10px 40px 10px 20px;
    cursor: pointer;
}

.menu-option:hover {
    background:#E8F0FE;
    color: #0056b3;
}
